/* This file is part of the Springlobby (GPL v2 or later), see COPYING */
#include "ToasterBoxWindowList.h"

#include <wx/bitmap.h>
#include <wx/dcclient.h>
#include <wx/dialog.h>
#include <wx/listimpl.cpp>
#include <wx/region.h>
#include <wx/string.h>
#include <wx/timer.h>
#include <wx/utils.h>

#include "ToasterBoxWindow.h"

WX_DEFINE_LIST(ToasterBoxWindowList)
