/* This file is part of the Springlobby (GPL v2 or later), see COPYING */
#ifndef FLAG_IMAGE_DATA_H
#define FLAG_IMAGE_DATA_H

// this header file is automatically generated with a script
#include "images/flags/AD.xpm"
#include "images/flags/AE.xpm"
#include "images/flags/AF.xpm"
#include "images/flags/AG.xpm"
#include "images/flags/AI.xpm"
#include "images/flags/AL.xpm"
#include "images/flags/AM.xpm"
#include "images/flags/AN.xpm"
#include "images/flags/AO.xpm"
#include "images/flags/AR.xpm"
#include "images/flags/AS.xpm"
#include "images/flags/AT.xpm"
#include "images/flags/AU.xpm"
#include "images/flags/AW.xpm"
#include "images/flags/AX.xpm"
#include "images/flags/AZ.xpm"
#include "images/flags/BA.xpm"
#include "images/flags/BB.xpm"
#include "images/flags/BD.xpm"
#include "images/flags/BE.xpm"
#include "images/flags/BF.xpm"
#include "images/flags/BG.xpm"
#include "images/flags/BH.xpm"
#include "images/flags/BI.xpm"
#include "images/flags/BJ.xpm"
#include "images/flags/BM.xpm"
#include "images/flags/BN.xpm"
#include "images/flags/BO.xpm"
#include "images/flags/BR.xpm"
#include "images/flags/BS.xpm"
#include "images/flags/BT.xpm"
#include "images/flags/BV.xpm"
#include "images/flags/BW.xpm"
#include "images/flags/BY.xpm"
#include "images/flags/BZ.xpm"
#include "images/flags/CA.xpm"
#include "images/flags/CC.xpm"
#include "images/flags/CD.xpm"
#include "images/flags/CF.xpm"
#include "images/flags/CG.xpm"
#include "images/flags/CH.xpm"
#include "images/flags/CI.xpm"
#include "images/flags/CK.xpm"
#include "images/flags/CL.xpm"
#include "images/flags/CM.xpm"
#include "images/flags/CN.xpm"
#include "images/flags/CO.xpm"
#include "images/flags/CR.xpm"
#include "images/flags/CS.xpm"
#include "images/flags/CU.xpm"
#include "images/flags/CV.xpm"
#include "images/flags/CX.xpm"
#include "images/flags/CY.xpm"
#include "images/flags/CZ.xpm"
#include "images/flags/DE.xpm"
#include "images/flags/DJ.xpm"
#include "images/flags/DK.xpm"
#include "images/flags/DM.xpm"
#include "images/flags/DO.xpm"
#include "images/flags/DZ.xpm"
#include "images/flags/EC.xpm"
#include "images/flags/EE.xpm"
#include "images/flags/EG.xpm"
#include "images/flags/EH.xpm"
#include "images/flags/ER.xpm"
#include "images/flags/ES.xpm"
#include "images/flags/ET.xpm"
#include "images/flags/EU.xpm"
#include "images/flags/FAM.xpm"
#include "images/flags/FI.xpm"
#include "images/flags/FJ.xpm"
#include "images/flags/FK.xpm"
#include "images/flags/FM.xpm"
#include "images/flags/FO.xpm"
#include "images/flags/FR.xpm"
#include "images/flags/GA.xpm"
#include "images/flags/GB.xpm"
#include "images/flags/GD.xpm"
#include "images/flags/GE.xpm"
#include "images/flags/GF.xpm"
#include "images/flags/GH.xpm"
#include "images/flags/GI.xpm"
#include "images/flags/GL.xpm"
#include "images/flags/GM.xpm"
#include "images/flags/GN.xpm"
#include "images/flags/GP.xpm"
#include "images/flags/GQ.xpm"
#include "images/flags/GR.xpm"
#include "images/flags/GS.xpm"
#include "images/flags/GT.xpm"
#include "images/flags/GU.xpm"
#include "images/flags/GW.xpm"
#include "images/flags/GY.xpm"
#include "images/flags/HK.xpm"
#include "images/flags/HM.xpm"
#include "images/flags/HN.xpm"
#include "images/flags/HR.xpm"
#include "images/flags/HT.xpm"
#include "images/flags/HU.xpm"
#include "images/flags/ID.xpm"
#include "images/flags/IE.xpm"
#include "images/flags/IL.xpm"
#include "images/flags/IM.xpm"
#include "images/flags/IN.xpm"
#include "images/flags/IO.xpm"
#include "images/flags/IQ.xpm"
#include "images/flags/IR.xpm"
#include "images/flags/IS.xpm"
#include "images/flags/IT.xpm"
#include "images/flags/JM.xpm"
#include "images/flags/JO.xpm"
#include "images/flags/JP.xpm"
#include "images/flags/KE.xpm"
#include "images/flags/KG.xpm"
#include "images/flags/KH.xpm"
#include "images/flags/KI.xpm"
#include "images/flags/KM.xpm"
#include "images/flags/KN.xpm"
#include "images/flags/KP.xpm"
#include "images/flags/KR.xpm"
#include "images/flags/KW.xpm"
#include "images/flags/KY.xpm"
#include "images/flags/KZ.xpm"
#include "images/flags/LA.xpm"
#include "images/flags/LB.xpm"
#include "images/flags/LC.xpm"
#include "images/flags/LI.xpm"
#include "images/flags/LK.xpm"
#include "images/flags/LR.xpm"
#include "images/flags/LS.xpm"
#include "images/flags/LT.xpm"
#include "images/flags/LU.xpm"
#include "images/flags/LV.xpm"
#include "images/flags/LY.xpm"
#include "images/flags/MA.xpm"
#include "images/flags/MC.xpm"
#include "images/flags/MD.xpm"
#include "images/flags/ME.xpm"
#include "images/flags/MG.xpm"
#include "images/flags/MH.xpm"
#include "images/flags/MK.xpm"
#include "images/flags/ML.xpm"
#include "images/flags/MM.xpm"
#include "images/flags/MN.xpm"
#include "images/flags/MO.xpm"
#include "images/flags/MP.xpm"
#include "images/flags/MQ.xpm"
#include "images/flags/MR.xpm"
#include "images/flags/MS.xpm"
#include "images/flags/MT.xpm"
#include "images/flags/MU.xpm"
#include "images/flags/MV.xpm"
#include "images/flags/MW.xpm"
#include "images/flags/MX.xpm"
#include "images/flags/MY.xpm"
#include "images/flags/MZ.xpm"
#include "images/flags/NA.xpm"
#include "images/flags/NC.xpm"
#include "images/flags/NE.xpm"
#include "images/flags/NF.xpm"
#include "images/flags/NG.xpm"
#include "images/flags/NI.xpm"
#include "images/flags/NL.xpm"
#include "images/flags/NO.xpm"
#include "images/flags/NP.xpm"
#include "images/flags/NR.xpm"
#include "images/flags/NU.xpm"
#include "images/flags/NZ.xpm"
#include "images/flags/OM.xpm"
#include "images/flags/PA.xpm"
#include "images/flags/PE.xpm"
#include "images/flags/PF.xpm"
#include "images/flags/PG.xpm"
#include "images/flags/PH.xpm"
#include "images/flags/PK.xpm"
#include "images/flags/PL.xpm"
#include "images/flags/PM.xpm"
#include "images/flags/PN.xpm"
#include "images/flags/PR.xpm"
#include "images/flags/PS.xpm"
#include "images/flags/PT.xpm"
#include "images/flags/PW.xpm"
#include "images/flags/PY.xpm"
#include "images/flags/QA.xpm"
#include "images/flags/RE.xpm"
#include "images/flags/RO.xpm"
#include "images/flags/RS.xpm"
#include "images/flags/RU.xpm"
#include "images/flags/RW.xpm"
#include "images/flags/SA.xpm"
#include "images/flags/SB.xpm"
#include "images/flags/SC.xpm"
#include "images/flags/SD.xpm"
#include "images/flags/SE.xpm"
#include "images/flags/SG.xpm"
#include "images/flags/SH.xpm"
#include "images/flags/SI.xpm"
#include "images/flags/SJ.xpm"
#include "images/flags/SK.xpm"
#include "images/flags/SL.xpm"
#include "images/flags/SM.xpm"
#include "images/flags/SN.xpm"
#include "images/flags/SO.xpm"
#include "images/flags/SR.xpm"
#include "images/flags/ST.xpm"
#include "images/flags/SV.xpm"
#include "images/flags/SY.xpm"
#include "images/flags/SZ.xpm"
#include "images/flags/TC.xpm"
#include "images/flags/TD.xpm"
#include "images/flags/TF.xpm"
#include "images/flags/TG.xpm"
#include "images/flags/TH.xpm"
#include "images/flags/TJ.xpm"
#include "images/flags/TK.xpm"
#include "images/flags/TL.xpm"
#include "images/flags/TM.xpm"
#include "images/flags/TN.xpm"
#include "images/flags/TO.xpm"
#include "images/flags/TR.xpm"
#include "images/flags/TT.xpm"
#include "images/flags/TV.xpm"
#include "images/flags/TW.xpm"
#include "images/flags/TZ.xpm"
#include "images/flags/UA.xpm"
#include "images/flags/UG.xpm"
#include "images/flags/UM.xpm"
#include "images/flags/US.xpm"
#include "images/flags/UY.xpm"
#include "images/flags/UZ.xpm"
#include "images/flags/VA.xpm"
#include "images/flags/VC.xpm"
#include "images/flags/VE.xpm"
#include "images/flags/VG.xpm"
#include "images/flags/VI.xpm"
#include "images/flags/VN.xpm"
#include "images/flags/VU.xpm"
#include "images/flags/WF.xpm"
#include "images/flags/WS.xpm"
#include "images/flags/YE.xpm"
#include "images/flags/YT.xpm"
#include "images/flags/ZA.xpm"
#include "images/flags/ZM.xpm"
#include "images/flags/ZW.xpm"

static const char* flag_str[] = {
    "AD",
    "AE",
    "AF",
    "AG",
    "AI",
    "AL",
    "AM",
    "AN",
    "AO",
    "AR",
    "AS",
    "AT",
    "AU",
    "AW",
    "AX",
    "AZ",
    "BA",
    "BB",
    "BD",
    "BE",
    "BF",
    "BG",
    "BH",
    "BI",
    "BJ",
    "BM",
    "BN",
    "BO",
    "BR",
    "BS",
    "BT",
    "BV",
    "BW",
    "BY",
    "BZ",
    "CA",
    "CC",
    "CD",
    "CF",
    "CG",
    "CH",
    "CI",
    "CK",
    "CL",
    "CM",
    "CN",
    "CO",
    "CR",
    "CS",
    "CU",
    "CV",
    "CX",
    "CY",
    "CZ",
    "DE",
    "DJ",
    "DK",
    "DM",
    "DO",
    "DZ",
    "EC",
    "EE",
    "EG",
    "EH",
    "ER",
    "ES",
    "ET",
    "EU",
    "FAM",
    "FI",
    "FJ",
    "FK",
    "FM",
    "FO",
    "FR",
    "GA",
    "GB",
    "GD",
    "GE",
    "GF",
    "GH",
    "GI",
    "GL",
    "GM",
    "GN",
    "GP",
    "GQ",
    "GR",
    "GS",
    "GT",
    "GU",
    "GW",
    "GY",
    "HK",
    "HM",
    "HN",
    "HR",
    "HT",
    "HU",
    "ID",
    "IE",
    "IL",
    "IM",
    "IN",
    "IO",
    "IQ",
    "IR",
    "IS",
    "IT",
    "JM",
    "JO",
    "JP",
    "KE",
    "KG",
    "KH",
    "KI",
    "KM",
    "KN",
    "KP",
    "KR",
    "KW",
    "KY",
    "KZ",
    "LA",
    "LB",
    "LC",
    "LI",
    "LK",
    "LR",
    "LS",
    "LT",
    "LU",
    "LV",
    "LY",
    "MA",
    "MC",
    "MD",
    "ME",
    "MG",
    "MH",
    "MK",
    "ML",
    "MM",
    "MN",
    "MO",
    "MP",
    "MQ",
    "MR",
    "MS",
    "MT",
    "MU",
    "MV",
    "MW",
    "MX",
    "MY",
    "MZ",
    "NA",
    "NC",
    "NE",
    "NF",
    "NG",
    "NI",
    "NL",
    "NO",
    "NP",
    "NR",
    "NU",
    "NZ",
    "OM",
    "PA",
    "PE",
    "PF",
    "PG",
    "PH",
    "PK",
    "PL",
    "PM",
    "PN",
    "PR",
    "PS",
    "PT",
    "PW",
    "PY",
    "QA",
    "RE",
    "RO",
    "RS",
    "RU",
    "RW",
    "SA",
    "SB",
    "SC",
    "SD",
    "SE",
    "SG",
    "SH",
    "SI",
    "SJ",
    "SK",
    "SL",
    "SM",
    "SN",
    "SO",
    "SR",
    "ST",
    "SV",
    "SY",
    "SZ",
    "TC",
    "TD",
    "TF",
    "TG",
    "TH",
    "TJ",
    "TK",
    "TL",
    "TM",
    "TN",
    "TO",
    "TR",
    "TT",
    "TV",
    "TW",
    "TZ",
    "UA",
    "UG",
    "UM",
    "US",
    "UY",
    "UZ",
    "VA",
    "VC",
    "VE",
    "VG",
    "VI",
    "VN",
    "VU",
    "WF",
    "WS",
    "YE",
    "YT",
    "ZA",
    "ZM",
    "ZW",
    0};

static const char* const* flag_xpm[] = {
    AD_xpm,
    AE_xpm,
    AF_xpm,
    AG_xpm,
    AI_xpm,
    AL_xpm,
    AM_xpm,
    AN_xpm,
    AO_xpm,
    AR_xpm,
    AS_xpm,
    AT_xpm,
    AU_xpm,
    AW_xpm,
    AX_xpm,
    AZ_xpm,
    BA_xpm,
    BB_xpm,
    BD_xpm,
    BE_xpm,
    BF_xpm,
    BG_xpm,
    BH_xpm,
    BI_xpm,
    BJ_xpm,
    BM_xpm,
    BN_xpm,
    BO_xpm,
    BR_xpm,
    BS_xpm,
    BT_xpm,
    BV_xpm,
    BW_xpm,
    BY_xpm,
    BZ_xpm,
    CA_xpm,
    CC_xpm,
    CD_xpm,
    CF_xpm,
    CG_xpm,
    CH_xpm,
    CI_xpm,
    CK_xpm,
    CL_xpm,
    CM_xpm,
    CN_xpm,
    CO_xpm,
    CR_xpm,
    CS_xpm,
    CU_xpm,
    CV_xpm,
    CX_xpm,
    CY_xpm,
    CZ_xpm,
    DE_xpm,
    DJ_xpm,
    DK_xpm,
    DM_xpm,
    DO_xpm,
    DZ_xpm,
    EC_xpm,
    EE_xpm,
    EG_xpm,
    EH_xpm,
    ER_xpm,
    ES_xpm,
    ET_xpm,
    EU_xpm,
    FAM_xpm,
    FI_xpm,
    FJ_xpm,
    FK_xpm,
    FM_xpm,
    FO_xpm,
    FR_xpm,
    GA_xpm,
    GB_xpm,
    GD_xpm,
    GE_xpm,
    GF_xpm,
    GH_xpm,
    GI_xpm,
    GL_xpm,
    GM_xpm,
    GN_xpm,
    GP_xpm,
    GQ_xpm,
    GR_xpm,
    GS_xpm,
    GT_xpm,
    GU_xpm,
    GW_xpm,
    GY_xpm,
    HK_xpm,
    HM_xpm,
    HN_xpm,
    HR_xpm,
    HT_xpm,
    HU_xpm,
    ID_xpm,
    IE_xpm,
    IL_xpm,
    IM_xpm,
    IN_xpm,
    IO_xpm,
    IQ_xpm,
    IR_xpm,
    IS_xpm,
    IT_xpm,
    JM_xpm,
    JO_xpm,
    JP_xpm,
    KE_xpm,
    KG_xpm,
    KH_xpm,
    KI_xpm,
    KM_xpm,
    KN_xpm,
    KP_xpm,
    KR_xpm,
    KW_xpm,
    KY_xpm,
    KZ_xpm,
    LA_xpm,
    LB_xpm,
    LC_xpm,
    LI_xpm,
    LK_xpm,
    LR_xpm,
    LS_xpm,
    LT_xpm,
    LU_xpm,
    LV_xpm,
    LY_xpm,
    MA_xpm,
    MC_xpm,
    MD_xpm,
    ME_xpm,
    MG_xpm,
    MH_xpm,
    MK_xpm,
    ML_xpm,
    MM_xpm,
    MN_xpm,
    MO_xpm,
    MP_xpm,
    MQ_xpm,
    MR_xpm,
    MS_xpm,
    MT_xpm,
    MU_xpm,
    MV_xpm,
    MW_xpm,
    MX_xpm,
    MY_xpm,
    MZ_xpm,
    NA_xpm,
    NC_xpm,
    NE_xpm,
    NF_xpm,
    NG_xpm,
    NI_xpm,
    NL_xpm,
    NO_xpm,
    NP_xpm,
    NR_xpm,
    NU_xpm,
    NZ_xpm,
    OM_xpm,
    PA_xpm,
    PE_xpm,
    PF_xpm,
    PG_xpm,
    PH_xpm,
    PK_xpm,
    PL_xpm,
    PM_xpm,
    PN_xpm,
    PR_xpm,
    PS_xpm,
    PT_xpm,
    PW_xpm,
    PY_xpm,
    QA_xpm,
    RE_xpm,
    RO_xpm,
    RS_xpm,
    RU_xpm,
    RW_xpm,
    SA_xpm,
    SB_xpm,
    SC_xpm,
    SD_xpm,
    SE_xpm,
    SG_xpm,
    SH_xpm,
    SI_xpm,
    SJ_xpm,
    SK_xpm,
    SL_xpm,
    SM_xpm,
    SN_xpm,
    SO_xpm,
    SR_xpm,
    ST_xpm,
    SV_xpm,
    SY_xpm,
    SZ_xpm,
    TC_xpm,
    TD_xpm,
    TF_xpm,
    TG_xpm,
    TH_xpm,
    TJ_xpm,
    TK_xpm,
    TL_xpm,
    TM_xpm,
    TN_xpm,
    TO_xpm,
    TR_xpm,
    TT_xpm,
    TV_xpm,
    TW_xpm,
    TZ_xpm,
    UA_xpm,
    UG_xpm,
    UM_xpm,
    US_xpm,
    UY_xpm,
    UZ_xpm,
    VA_xpm,
    VC_xpm,
    VE_xpm,
    VG_xpm,
    VI_xpm,
    VN_xpm,
    VU_xpm,
    WF_xpm,
    WS_xpm,
    YE_xpm,
    YT_xpm,
    ZA_xpm,
    ZM_xpm,
    ZW_xpm,
    0};

#endif //FLAG_IMAGE_DATA_H
